<?php
/*
Plugin Name: David's Gunning-Fog Text Analyzer
Plugin URI: http://www.davidgagne.net/tags/code
Description: This plugin runs a Gunning-Fog analysis of a post's content.
Author: David Vincent Gagne
Version: 0.5
Author URI: http://www.davidgagne.net/

Instructions:

To display the Gunning-Fog analysis of a post's content, include the following line in your template:

<?php dg_GF($post->post_content); ?>

*/ 

/****************************
     This function does the dirty work ...
****************************/
function dg_GF( $content ){
     if ( $_GET["gf"] == 1 ){
          $gfdebug = 1;
     }
     else{
          $gfdebug = 0;
     }
     /****************************
          initialize return variable
     ****************************/
     $newcontent = "";
     /****************************
          rip out the HTML
     ****************************/
     $analysis = strip_tags($content);
     $analysis = html_entity_decode($analysis);
#     echo "<b>$analysis</b>";
     /****************************
          rip out any numbers
     ****************************/
     $analysis = ereg_replace("[0-9]", "", $analysis);
     /****************************
          rip out any line feeds
     ****************************/
     $analysis = str_replace("\r\n", " ", $analysis);
     /****************************
          rip out non-alpha characters
     ****************************/
     $badguys = array(";", ",", "(", ")", "@", "$", "%", "^", "&", "*", "-", "+", "=", "'", "\"", "[", "]", ":", "", "", "/", "\\");
     $analysis = str_replace($badguys, "", $analysis);
     /****************************
          count the sentences (anything ending in a . or ! or ?
     ****************************/
     $sentences = count(preg_split("/(\!|\.|\?)/",$analysis)) - 1;
     /****************************
          remove the sentence-enders
     ****************************/
     $badguys = array(".", "!", "?");
     $analysis = str_replace($badguys, "", $analysis);
     $analysis = preg_replace('/\s\s+/', ' ', $analysis);
     /****************************
          count the words left
     ****************************/
     $words = count(explode(" ", $analysis)) - 1;
     /****************************
          store the words in an array
     ****************************/
     $wordbatch = explode(" ", $analysis);
     /****************************
          initialize the $complexwords var
     ****************************/
     $complexwords = 0;
     /****************************
          consider anything longer than 6 characters to be "complex"
     ****************************/
     for ($i = 0; $i < $words; $i++){
          if ( count_syllables( $wordbatch[$i] ) > 3 ){
#               echo "<br />" . $wordbatch[$i];
               $complexwords++;
          }
     }
#     $newcontent .= "<br /><br />[" . $analysis . "]";
#     $newcontent .= "<h3>" . $words . "</h3>";
#     $newcontent .= "<h3>" . $sentences . "</h3>";
#     $newcontent .= "<h3>" . $complexwords . "</h3>";
     $gf = .4 * ( ($words / $sentences) + 100 * ($complexwords/$words) );
     if ( $sentences > 0 && $words > 0 ){
          $gf = "The (approximate) <a href=\"http://www.davidgagne.net/tags/wordpress\">Gunning-Fog Index</a> for this post is <a rel=\"nofollow\" title=\"details\" href=\"?" . str_replace("&rw=1", "", str_replace("&cw=1", "", str_replace("&gf=1", "", $_SERVER['QUERY_STRING']))) . "&gf=1\">" . number_format($gf, 2) . "</a>.";
     }
     else{
          $gf = "<a href=\"http://www.davidgagne.net/tags/wordpress\">Gunning-Fog Index</a> unavailable for this content ...";
     }
     $newcontent .= "<br /><br /><b>" . $gf . "</b>";
     if ( $gfdebug == 1 ){
          $newcontent .= "<p>The Gunning-Fog Index for a page is calculated using the formula:</p>";
          $newcontent .= "<div style=\"text-align:left;\">.4 * ((words / sentences) + 100 * (complexwords/words)) <div style=\"text-align:right;font-style:italic;\">from <a rel=\"nofollow\" href=\"http://en.wikipedia.org/wiki/Gunning-Fog_Index\">wikipedia</a></div></div>";

          $worddisplay = ( $words > 1 ) ? "words" : "word";
          $worddisplay = ( $words == 0 ) ? $worddisplay : "<a href=\"?" . str_replace("&cw=1", "", $_SERVER['QUERY_STRING']) . "&rw=1\">" . $worddisplay . "</a>";
          $sentencedisplay = ( $sentences > 1 ) ? "sentences" : "sentence";
          $complexwordsdisplay = ( $complexwords > 1 ) ? "\"complex\" words" : "\"complex\" word";
          $complexwordsdisplay = ( $complexwords == 0 ) ? $complexwordsdisplay : "<a href=\"?" . str_replace("&rw=1", "", $_SERVER['QUERY_STRING']) . "&cw=1\">" . $complexwordsdisplay . "</a>";

          $newcontent .= "<p>This page has <b>$words</b> $worddisplay, <b>$sentences</b> $sentencedisplay, and <b>$complexwords</b> $complexwordsdisplay.</p>";
          if ( $_GET["cw"] == 1 ){
               $newcontent .= "<p style=\"text-align:left;\">The \"complex\" words are those with three or more syllables:<ol>";
               $complexwords = 0;
               for ($i = 0; $i < $words; $i++){
                    if ( count_syllables( $wordbatch[$i] ) > 3 ){
                         $newcontent .= "<li>" . strtolower($wordbatch[$i]) . "</li>";
                    }
               }
               $newcontent .= "</ol>";
          }
          if ( $_GET["rw"] == 1 ){
               $newcontent .= "<p style=\"text-align:left;\">Words:<ol>";
               for ($i = 0; $i < $words; $i++){
                    $newcontent .= "<li>" . strtolower($wordbatch[$i]) . "</li>";
               }
               $newcontent .= "</ol>";
          }
          $newcontent = rtrim($newcontent, ", ");
     }
     echo $newcontent;
}

/****************************
     Count_syllables function taken from http://www.ilovejackdaniels.com/php/gunning-fog-function/
****************************/
     function count_syllables($word){      
          $subsyl = array('cial','tia','cius','cious','giu','ion','iou','sia$','.ely$');

          $addsyl = array('ia','riet','dien','iu','io','ii','[aeiouym]bl$','[aeiou]{3}','^mc','ism$','([^aeiouy])\1l$','[^l]lien','^coa[dglx].','[^gq]ua[^auieo]','dnt$'); 

     // Based on Greg Fast's Perl module Lingua::EN::Syllables
          $word = preg_replace('/[^a-z]/is', '', strtolower($word));
          $word_parts = preg_split('/[^aeiouy]+/', $word);
          foreach ($word_parts as $key => $value){
               if ($value <> '') {
                    $valid_word_parts[] = $value;
               }
          }
      
          foreach ($subsyl as $syl) {
          if (strpos($word, $syl) !== false) {
               $syllables--;
               }
          }
          foreach ($addsyl as $syl) {
               if (strpos($word, $syl) !== false) {
                    $syllables++;
               }
          }
          if (strlen($word) == 1) {
               $syllables++;
          }
          $syllables += count($valid_word_parts);
          $syllables = ($syllables == 0) ? 1 : $syllables;
          return $syllables;
     }
?>